#**************************************************************************** 
#** 
#** File : /cdimage/units/UAL0108/UAL0108_script.lua 
#** Author(s): Ebola Soup, Resin_Smoker 
#** 
#** Summary : Aeon Medium Assault Walker 
#** 
#** Copyright  2009, 4th Dimension 
#**************************************************************************** 

local AWalkingLandUnit = import('/lua/aeonunits.lua').AWalkingLandUnit 
local ADFLaserLightWeapon = import('/lua/aeonweapons.lua').ADFLaserLightWeapon
local AAAZealotMissileWeapon = import('/lua/aeonweapons.lua').AAAZealotMissileWeapon 

est0002 = Class(AWalkingLandUnit) { 
    Weapons = { 
        MainGun = Class(ADFLaserLightWeapon) {}, 
        RocketBackpack = Class(AAAZealotMissileWeapon) {},
    },
    
    OnStopBeingBuilt = function(self,builder,layer) 
        AWalkingLandUnit.OnStopBeingBuilt(self,builder,layer)      
        ### Phase Global Variables 
        self.BP = self:GetBlueprint() 
        self.PhaseEnabled = false
        self.PriorStatus = false 
        self:SetMaintenanceConsumptionInactive()
        if self.BP.Defense.Phasing.StartsOn == true then
            self:SetScriptBit('RULEUTC_ShieldToggle', true) 
        end        
        self:ForkThread(self.EconomyHeartBeat)                                                                       
    end,
    
    EconomyHeartBeat = function(self)
        local cost = self.BP.Economy.MaintenanceConsumptionPerSecondEnergy           
        while not self:IsDead() do    
            local econ = self:GetAIBrain():GetEconomyStored('Energy')           
            if econ < cost and self.PhaseEnabled == true then
                self.PriorStatus = true
                self:ForkThread(self.PhaseDisable) 
            elseif self.PriorStatus == true and self.PhaseEnabled == false and econ > cost then
                self.PriorStatus = false
                self:ForkThread(self.PhaseEnable)
            end       
            ### Short delay between checks
            WaitSeconds(Random(1, 3))         
        end
    end, 
            
    OnScriptBitSet = function(self, bit)
        if bit == 0 then
            self:ForkThread(self.PhaseEnable)
        end
        AWalkingLandUnit.OnScriptBitSet(self, bit)
    end,     
     
    OnScriptBitClear = function(self, bit)
       if bit == 0 then
           self.PriorStatus = false
           self:ForkThread(self.PhaseDisable)      
       end
       AWalkingLandUnit.OnScriptBitClear(self, bit)
    end,    
    
    PhaseEnable = function(self)
        if not self:IsDead() and not self.PhaseEnabled == true then 
            self.PhaseEnabled = true
            self:SetMaintenanceConsumptionActive() 
            self:SetMesh(self.BP.Defense.Phasing.PhaseMesh, true)  
        end
    end,    
    
    PhaseDisable = function(self)
        if not self:IsDead() and not self.PhaseEnabled == false then    
            self.PhaseEnabled = false    
            self:SetMaintenanceConsumptionInactive() 
            self:SetMesh(self.BP.Defense.Phasing.StandardMesh, true)            
        end
    end,
    
    OnCollisionCheck = function(self, other, firingWeapon)
        if not self:IsDead() and self.PhaseEnabled == true then
            if EntityCategoryContains(categories.PROJECTILE, other) then  
                local random = Random(1,100) 
                ### Allows % of projectiles to pass 
                if random <= self.BP.Defense.Phasing.PhasePercent then   
                    ### Returning false allows the projectile to pass thru
                    return false       
                else 
                    ### Projectile impacts normally 
                    return true  
                end
            end
        else
            ### Projectile impacts normally 
            return true            
        end
        AWalkingLandUnit.OnCollisionCheck(self, other, firingWeapon)  
    end,                      
} 
TypeClass = est0002